@@ -27,12 +27,6 @@
 
 using namespace CalendarSupport;
 
-static QString cleanString(const QString &instr)
-{
-    QString ret = instr;
-    return ret.replace(QLatin1Char('\n'), QLatin1Char(' '));
-}
-
 /**************************************************************
  *           Print Incidence
  **************************************************************/
@@ -53,7 +47,7 @@ QWidget *CalPrintIncidence::createConfigWidget(QWidget *w)
 
 void CalPrintIncidence::readSettingsWidget()
 {
-    auto *cfg = dynamic_cast<CalPrintIncidenceConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintIncidenceConfig *>((QWidget *)mConfigWidget);
     if (cfg) {
         mUseColors = cfg->mColors->isChecked();
         mPrintFooter = cfg->mPrintFooter->isChecked();
@@ -67,7 +61,7 @@ void CalPrintIncidence::readSettingsWidget()
 
 void CalPrintIncidence::setSettingsWidget()
 {
-    auto *cfg = dynamic_cast<CalPrintIncidenceConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintIncidenceConfig *>((QWidget *)mConfigWidget);
     if (cfg) {
         cfg->mColors->setChecked(mUseColors);
         cfg->mPrintFooter->setChecked(mPrintFooter);
@@ -187,15 +181,10 @@ protected:
     }
 };
 
-int CalPrintIncidence::printCaptionAndText(QPainter &p,
-                                           const QRect &box,
-                                           const QString &caption,
-                                           const QString &text,
-                                           const QFont &captionFont,
-                                           const QFont &textFont)
+int CalPrintIncidence::printCaptionAndText(QPainter &p, QRect box, const QString &caption, const QString &text, const QFont &captionFont, const QFont &textFont)
 {
     QFontMetrics captionFM(captionFont);
-    int textWd = captionFM.boundingRect(caption).width();
+    int textWd = captionFM.horizontalAdvance(caption);
     QRect textRect(box);
 
     QFont oldFont(p.font());
@@ -265,7 +254,7 @@ void CalPrintIncidence::print(QPainter &p, int width, int height)
         QRect box(0, 0, width, height);
         QRect titleBox(box);
         titleBox.setHeight(headerHeight());
-        QColor headerColor = categoryBgColor(*it);
+        QColor headerColor = mUseColors ? categoryBgColor(*it) : QColor();
         // Draw summary as header, no small calendars in title bar, expand height if needed
         int titleBottom = drawHeader(p, (*it)->summary(), QDate(), QDate(), titleBox, true, headerColor);
         titleBox.setBottom(titleBottom);
@@ -460,7 +449,9 @@ void CalPrintIncidence::print(QPainter &p, int width, int height)
                                                        /*expand=*/false,
                                                        captionFont,
                                                        textFont);
-                drawNoteLines(p, notesBox, notesPosition);
+                if (mShowNoteLines) {
+                    drawNoteLines(p, notesBox, notesPosition);
+                }
             } else {
                 QString subitemCaption;
                 if (relations.isEmpty()) {
@@ -474,7 +465,7 @@ void CalPrintIncidence::print(QPainter &p, int width, int height)
                 QString statusString;
                 QString datesString;
                 int count = 0;
-                for (const Akonadi::Item &item : qAsConst(relations)) {
+                for (const Akonadi::Item &item : std::as_const(relations)) {
                     KCalendarCore::Todo::Ptr todo = CalendarSupport::todo(item);
                     ++count;
                     if (!todo) { // defensive, skip any zero pointers
@@ -661,7 +652,7 @@ QWidget *CalPrintDay::createConfigWidget(QWidget *w)
 
 void CalPrintDay::readSettingsWidget()
 {
-    auto *cfg = dynamic_cast<CalPrintDayConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintDayConfig *>((QWidget *)mConfigWidget);
     if (cfg) {
         mFromDate = cfg->mFromDate->date();
         mToDate = cfg->mToDate->date();
@@ -679,6 +670,7 @@ void CalPrintDay::readSettingsWidget()
         mIncludeAllEvents = cfg->mIncludeAllEvents->isChecked();
 
         mIncludeDescription = cfg->mIncludeDescription->isChecked();
+        mIncludeCategories = cfg->mIncludeCategories->isChecked();
         mSingleLineLimit = cfg->mSingleLineLimit->isChecked();
         mIncludeTodos = cfg->mIncludeTodos->isChecked();
         mUseColors = cfg->mColors->isChecked();
@@ -692,7 +684,7 @@ void CalPrintDay::readSettingsWidget()
 
 void CalPrintDay::setSettingsWidget()
 {
-    auto *cfg = dynamic_cast<CalPrintDayConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintDayConfig *>((QWidget *)mConfigWidget);
     if (cfg) {
         cfg->mFromDate->setDate(mFromDate);
         cfg->mToDate->setDate(mToDate);
@@ -706,6 +698,7 @@ void CalPrintDay::setSettingsWidget()
         cfg->mIncludeAllEvents->setChecked(mIncludeAllEvents);
 
         cfg->mIncludeDescription->setChecked(mIncludeDescription);
+        cfg->mIncludeCategories->setChecked(mIncludeCategories);
         cfg->mSingleLineLimit->setChecked(mSingleLineLimit);
         cfg->mIncludeTodos->setChecked(mIncludeTodos);
         cfg->mColors->setChecked(mUseColors);
@@ -728,14 +721,13 @@ void CalPrintDay::loadConfig()
         mStartTime = grp.readEntry("Start time", startTm).time();
         mEndTime = grp.readEntry("End time", endTm).time();
         mIncludeDescription = grp.readEntry("Include description", false);
+        mIncludeCategories = grp.readEntry("Include categories", false);
         mIncludeTodos = grp.readEntry("Include todos", false);
         mIncludeAllEvents = grp.readEntry("Include all events", false);
         mDayPrintType = static_cast<eDayPrintType>(grp.readEntry("Print type", static_cast<int>(Timetable)));
         mSingleLineLimit = grp.readEntry("Single line limit", false);
         mShowNoteLines = grp.readEntry("Note Lines", false);
         mExcludeTime = grp.readEntry("Exclude time", false);
-        mExcludeConfidential = grp.readEntry("Exclude confidential", true);
-        mExcludePrivate = grp.readEntry("Exclude private", true);
     }
     setSettingsWidget();
 }
@@ -751,21 +743,20 @@ void CalPrintDay::saveConfig()
         dt.setTime(mEndTime);
         grp.writeEntry("End time", dt);
         grp.writeEntry("Include description", mIncludeDescription);
+        grp.writeEntry("Include categories", mIncludeCategories);
         grp.writeEntry("Include todos", mIncludeTodos);
         grp.writeEntry("Include all events", mIncludeAllEvents);
         grp.writeEntry("Print type", int(mDayPrintType));
         grp.writeEntry("Single line limit", mSingleLineLimit);
         grp.writeEntry("Note Lines", mShowNoteLines);
         grp.writeEntry("Exclude time", mExcludeTime);
-        grp.writeEntry("Exclude confidential", mExcludeConfidential);
-        grp.writeEntry("Exclude private", mExcludePrivate);
     }
 }
 
 void CalPrintDay::setDateRange(const QDate &from, const QDate &to)
 {
     CalPrintPluginBase::setDateRange(from, to);
-    auto *cfg = dynamic_cast<CalPrintDayConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintDayConfig *>((QWidget *)mConfigWidget);
     if (cfg) {
         cfg->mFromDate->setDate(from);
         cfg->mToDate->setDate(to);
@@ -779,22 +770,22 @@ void CalPrintDay::print(QPainter &p, int width, int height)
     QRect headerBox(0, 0, width, headerHeight());
     QRect footerBox(0, height - footerHeight(), width, footerHeight());
     height -= footerHeight();
+    QRect daysBox(headerBox);
+    daysBox.setTop(headerBox.bottom() + padding());
+    daysBox.setBottom(height);
 
     auto local = QLocale::system();
 
     switch (mDayPrintType) {
     case Filofax:
     case SingleTimetable: {
-        QRect daysBox(headerBox);
-        daysBox.setTop(headerBox.bottom() + padding());
-        daysBox.setBottom(height);
         QString line1 = local.toString(mFromDate, QLocale::ShortFormat);
         QString line2 = local.toString(mToDate, QLocale::ShortFormat);
         QString title;
-        if (orientation() == QPageLayout::Landscape) {
-            title = i18nc("date from-to", "%1 - %2", line1, line2);
+        if (mFromDate == mToDate) {
+            title = line1;
         } else {
-            title = i18nc("date from-\nto", "%1 -\n%2", line1, line2);
+            title =  i18nc("date from-to", "%1\u2013%2", line1, line2);
         }
         drawHeader(p, title, mFromDate, QDate(), headerBox);
         if (mDayPrintType == Filofax) {
@@ -805,10 +796,8 @@ void CalPrintDay::print(QPainter &p, int width, int height)
                      mEndTime,
                      daysBox,
                      mSingleLineLimit,
-                     mShowNoteLines,
                      mIncludeDescription,
-                     mExcludeConfidential,
-                     mExcludePrivate);
+                     mIncludeCategories);
         } else if (mDayPrintType == SingleTimetable) {
             drawTimeTable(p,
                           mFromDate,
@@ -818,9 +807,8 @@ void CalPrintDay::print(QPainter &p, int width, int height)
                           mEndTime,
                           daysBox,
                           mIncludeDescription,
-                          mExcludeTime,
-                          mExcludeConfidential,
-                          mExcludePrivate);
+                          mIncludeCategories,
+                          mExcludeTime);
         }
         if (mPrintFooter) {
             drawFooter(p, footerBox);
@@ -842,93 +830,16 @@ void CalPrintDay::print(QPainter &p, int width, int height)
             }
 
             drawHeader(p, local.toString(curDay, QLocale::ShortFormat), curDay, QDate(), headerBox);
-            KCalendarCore::Event::List eventList =
-                mCalendar->events(curDay, QTimeZone::systemTimeZone(), KCalendarCore::EventSortStartDate, KCalendarCore::SortDirectionAscending);
-
-            // split out the all day events as they will be printed in a separate box
-            KCalendarCore::Event::List alldayEvents, timedEvents;
-            for (const KCalendarCore::Event::Ptr &event : qAsConst(eventList)) {
-                if (!event
-                    || (mExcludeConfidential && event->secrecy() == KCalendarCore::Incidence::SecrecyConfidential)
-                    || (mExcludePrivate && event->secrecy() == KCalendarCore::Incidence::SecrecyPrivate)) {
-                    continue;
-                }
-                if (event->allDay()) {
-                    alldayEvents.append(event);
-                } else {
-                    timedEvents.append(event);
-                }
-            }
-
-            int fontSize = 11;
-            QFont textFont(QStringLiteral("sans-serif"), fontSize, QFont::Normal);
-            p.setFont(textFont);
-            int lineSpacing = p.fontMetrics().lineSpacing();
-
-            int maxAllDayEvents = 8; // the max we allow to be printed, sorry.
-            int allDayHeight = qMin(alldayEvents.count(), maxAllDayEvents) * lineSpacing;
-            allDayHeight = qMax(allDayHeight, (5 * lineSpacing)) + (2 * padding());
-            QRect allDayBox(TIMELINE_WIDTH + padding(), headerBox.bottom() + padding(), width - TIMELINE_WIDTH - padding(), allDayHeight);
-            if (!alldayEvents.isEmpty()) {
-                // draw the side bar for all-day events
-                QFont oldFont(p.font());
-                p.setFont(QFont(QStringLiteral("sans-serif"), 9, QFont::Normal));
-                drawVerticalBox(p,
-                                BOX_BORDER_WIDTH,
-                                QRect(0, headerBox.bottom() + padding(), TIMELINE_WIDTH, allDayHeight),
-                                i18n("Today's Events"),
-                                Qt::AlignHCenter | Qt::AlignVCenter | Qt::TextWordWrap);
-                p.setFont(oldFont);
-
-                // now draw at most maxAllDayEvents in the all-day box
-                drawBox(p, BOX_BORDER_WIDTH, allDayBox);
-
-                QRect eventBox(allDayBox);
-                eventBox.setLeft(TIMELINE_WIDTH + (2 * padding()));
-                eventBox.setTop(eventBox.top() + padding());
-                eventBox.setBottom(eventBox.top() + lineSpacing);
-                int count = 0;
-                for (const KCalendarCore::Event::Ptr &event : qAsConst(alldayEvents)) {
-                    if (count == maxAllDayEvents) {
-                        break;
-                    }
-                    count++;
-                    QString str;
-                    if (event->location().isEmpty()) {
-                        str = cleanString(event->summary());
-                    } else {
-                        str = i18nc("summary, location", "%1, %2", cleanString(event->summary()), cleanString(event->location()));
-                    }
-                    printEventString(p, eventBox, str);
-                    eventBox.setTop(eventBox.bottom());
-                    eventBox.setBottom(eventBox.top() + lineSpacing);
-                }
-            } else {
-                allDayBox.setBottom(headerBox.bottom());
-            }
-
-            QRect dayBox(allDayBox);
-            dayBox.setTop(allDayBox.bottom() + padding());
-            dayBox.setBottom(height);
-            QList<QDate> workDays = CalendarSupport::workDays(curDay, curDay);
-            drawAgendaDayBox(p,
-                             timedEvents,
-                             curDay,
-                             mIncludeAllEvents,
-                             curStartTime,
-                             curEndTime,
-                             dayBox,
-                             mIncludeDescription,
-                             mExcludeTime,
-                             mExcludeConfidential,
-                             mExcludePrivate,
-                             workDays);
-
-            QRect tlBox(dayBox);
-            tlBox.setLeft(0);
-            tlBox.setWidth(TIMELINE_WIDTH);
-            drawTimeLine(p, curStartTime, curEndTime, tlBox);
-
+            drawTimeTable(p,
+                          curDay,
+                          curDay,
+                          mIncludeAllEvents,
+                          mStartTime,
+                          mEndTime,
+                          daysBox,
+                          mIncludeDescription,
+                          mIncludeCategories,
+                          mExcludeTime);
             if (mPrintFooter) {
                 drawFooter(p, footerBox);
             }
@@ -961,7 +872,7 @@ QWidget *CalPrintWeek::createConfigWidget(QWidget *w)
 
 void CalPrintWeek::readSettingsWidget()
 {
-    auto *cfg = dynamic_cast<CalPrintWeekConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintWeekConfig *>((QWidget *)mConfigWidget);
     if (cfg) {
         mFromDate = cfg->mFromDate->date();
         mToDate = cfg->mToDate->date();
@@ -978,6 +889,7 @@ void CalPrintWeek::readSettingsWidget()
 
         mStartTime = cfg->mFromTime->time();
         mEndTime = cfg->mToTime->time();
+        mIncludeAllEvents = cfg->mIncludeAllEvents->isChecked();
 
         mShowNoteLines = cfg->mShowNoteLines->isChecked();
         mSingleLineLimit = cfg->mSingleLineLimit->isChecked();
@@ -985,6 +897,7 @@ void CalPrintWeek::readSettingsWidget()
         mUseColors = cfg->mColors->isChecked();
         mPrintFooter = cfg->mPrintFooter->isChecked();
         mIncludeDescription = cfg->mIncludeDescription->isChecked();
+        mIncludeCategories = cfg->mIncludeCategories->isChecked();
         mExcludeTime = cfg->mExcludeTime->isChecked();
         mExcludeConfidential = cfg->mExcludeConfidential->isChecked();
         mExcludePrivate = cfg->mExcludePrivate->isChecked();
@@ -993,7 +906,7 @@ void CalPrintWeek::readSettingsWidget()
 
 void CalPrintWeek::setSettingsWidget()
 {
-    auto *cfg = dynamic_cast<CalPrintWeekConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintWeekConfig *>((QWidget *)mConfigWidget);
     if (cfg) {
         cfg->mFromDate->setDate(mFromDate);
         cfg->mToDate->setDate(mToDate);
@@ -1004,6 +917,7 @@ void CalPrintWeek::setSettingsWidget()
 
         cfg->mFromTime->setTime(mStartTime);
         cfg->mToTime->setTime(mEndTime);
+        cfg->mIncludeAllEvents->setChecked(mIncludeAllEvents);
 
         cfg->mShowNoteLines->setChecked(mShowNoteLines);
         cfg->mSingleLineLimit->setChecked(mSingleLineLimit);
@@ -1011,6 +925,7 @@ void CalPrintWeek::setSettingsWidget()
         cfg->mColors->setChecked(mUseColors);
         cfg->mPrintFooter->setChecked(mPrintFooter);
         cfg->mIncludeDescription->setChecked(mIncludeDescription);
+        cfg->mIncludeCategories->setChecked(mIncludeCategories);
         cfg->mExcludeTime->setChecked(mExcludeTime);
         cfg->mExcludeConfidential->setChecked(mExcludeConfidential);
         cfg->mExcludePrivate->setChecked(mExcludePrivate);
@@ -1030,11 +945,11 @@ void CalPrintWeek::loadConfig()
         mShowNoteLines = grp.readEntry("Note Lines", false);
         mSingleLineLimit = grp.readEntry("Single line limit", false);
         mIncludeTodos = grp.readEntry("Include todos", false);
+        mIncludeAllEvents = grp.readEntry("Include all events", false);
         mWeekPrintType = (eWeekPrintType)(grp.readEntry("Print type", (int)Filofax));
         mIncludeDescription = grp.readEntry("Include Description", false);
+        mIncludeCategories = grp.readEntry("Include categories", false);
         mExcludeTime = grp.readEntry("Exclude Time", false);
-        mExcludeConfidential = grp.readEntry("Exclude confidential", true);
-        mExcludePrivate = grp.readEntry("Exclude private", true);
     }
     setSettingsWidget();
 }
@@ -1052,11 +967,11 @@ void CalPrintWeek::saveConfig()
         grp.writeEntry("Note Lines", mShowNoteLines);
         grp.writeEntry("Single line limit", mSingleLineLimit);
         grp.writeEntry("Include todos", mIncludeTodos);
+        grp.writeEntry("Include all events", mIncludeAllEvents);
         grp.writeEntry("Print type", int(mWeekPrintType));
         grp.writeEntry("Include Description", mIncludeDescription);
+        grp.writeEntry("Include categories", mIncludeCategories);
         grp.writeEntry("Exclude Time", mExcludeTime);
-        grp.writeEntry("Exclude confidential", mExcludeConfidential);
-        grp.writeEntry("Exclude private", mExcludePrivate);
     }
 }
 
@@ -1074,7 +989,7 @@ QPageLayout::Orientation CalPrintWeek::defaultOrientation() const
 void CalPrintWeek::setDateRange(const QDate &from, const QDate &to)
 {
     CalPrintPluginBase::setDateRange(from, to);
-    auto *cfg = dynamic_cast<CalPrintWeekConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintWeekConfig *>((QWidget *)mConfigWidget);
     if (cfg) {
         cfg->mFromDate->setDate(from);
         cfg->mToDate->setDate(to);
@@ -1108,14 +1023,11 @@ void CalPrintWeek::print(QPainter &p, int width, int height)
         do {
             line1 = local.toString(curWeek.addDays(-6), QLocale::ShortFormat);
             line2 = local.toString(curWeek, QLocale::ShortFormat);
-            if (orientation() == QPageLayout::Landscape) {
-                title = i18nc("date from-to", "%1 - %2", line1, line2);
-            } else {
-                title = i18nc("date from-\nto", "%1 -\n%2", line1, line2);
-            }
+            title =  i18nc("date from-to", "%1\u2013%2", line1, line2);
             drawHeader(p, title, curWeek.addDays(-6), QDate(), headerBox);
 
-            drawWeek(p, curWeek, mStartTime, mEndTime, weekBox, mSingleLineLimit, mShowNoteLines, mIncludeDescription, mExcludeConfidential, mExcludePrivate);
+            drawWeek(p, curWeek, mStartTime, mEndTime, weekBox, mSingleLineLimit,
+                      mIncludeDescription, mIncludeCategories);
 
             if (mPrintFooter) {
                 drawFooter(p, footerBox);
@@ -1134,13 +1046,14 @@ void CalPrintWeek::print(QPainter &p, int width, int height)
             line1 = local.toString(curWeek.addDays(-6), QLocale::ShortFormat);
             line2 = local.toString(curWeek, QLocale::ShortFormat);
             if (orientation() == QPageLayout::Landscape) {
-                title = i18nc("date from - to (week number)", "%1 - %2 (Week %3)", line1, line2, curWeek.weekNumber());
+                title = i18nc("date from - to (week number)", "%1\u2013%2 (Week %3)", line1, line2, curWeek.weekNumber());
             } else {
-                title = i18nc("date from -\nto (week number)", "%1 -\n%2 (Week %3)", line1, line2, curWeek.weekNumber());
+                title = i18nc("date from - to\\n(week number)", "%1\u2013%2\n(Week %3)", line1, line2, curWeek.weekNumber());
             }
             drawHeader(p, title, curWeek, QDate(), headerBox);
 
-            drawTimeTable(p, fromWeek, curWeek, false, mStartTime, mEndTime, weekBox, mIncludeDescription, mExcludeTime, mExcludeConfidential, mExcludePrivate);
+            drawTimeTable(p, fromWeek, curWeek, mIncludeAllEvents, mStartTime, mEndTime, weekBox, mIncludeDescription,
+                           mIncludeCategories, mExcludeTime);
 
             if (mPrintFooter) {
                 drawFooter(p, footerBox);
@@ -1165,7 +1078,8 @@ void CalPrintWeek::print(QPainter &p, int width, int height)
             int hh = headerHeight();
 
             drawSplitHeaderRight(p, fromWeek, curWeek, QDate(), width, hh);
-            drawTimeTable(p, fromWeek, endLeft, false, mStartTime, mEndTime, weekBox, mIncludeDescription, mExcludeTime, mExcludeConfidential, mExcludePrivate);
+            drawTimeTable(p, fromWeek, endLeft, mIncludeAllEvents, mStartTime, mEndTime, weekBox, mIncludeDescription,
+                           mIncludeCategories, mExcludeTime);
             if (mPrintFooter) {
                 drawFooter(p, footerBox);
             }
@@ -1174,14 +1088,13 @@ void CalPrintWeek::print(QPainter &p, int width, int height)
             drawTimeTable(p,
                           endLeft.addDays(1),
                           curWeek,
-                          false,
+                          mIncludeAllEvents,
                           mStartTime,
                           mEndTime,
                           weekBox1,
                           mIncludeDescription,
-                          mExcludeTime,
-                          mExcludeConfidential,
-                          mExcludePrivate);
+                          mIncludeCategories,
+                          mExcludeTime);
 
             if (mPrintFooter) {
                 drawFooter(p, footerBox);
@@ -1218,7 +1131,7 @@ QWidget *CalPrintMonth::createConfigWidget(QWidget *w)
 
 void CalPrintMonth::readSettingsWidget()
 {
-    auto *cfg = dynamic_cast<CalPrintMonthConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintMonthConfig *>((QWidget *)mConfigWidget);
 
     if (cfg) {
         mFromDate = QDate(cfg->mFromYear->value(), cfg->mFromMonth->currentIndex() + 1, 1);
@@ -1233,6 +1146,7 @@ void CalPrintMonth::readSettingsWidget()
         mUseColors = cfg->mColors->isChecked();
         mPrintFooter = cfg->mPrintFooter->isChecked();
         mIncludeDescription = cfg->mIncludeDescription->isChecked();
+        mIncludeCategories = cfg->mIncludeCategories->isChecked();
         mExcludeConfidential = cfg->mExcludeConfidential->isChecked();
         mExcludePrivate = cfg->mExcludePrivate->isChecked();
     }
@@ -1240,7 +1154,7 @@ void CalPrintMonth::readSettingsWidget()
 
 void CalPrintMonth::setSettingsWidget()
 {
-    auto *cfg = dynamic_cast<CalPrintMonthConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintMonthConfig *>((QWidget *)mConfigWidget);
 
     if (cfg) {
         setDateRange(mFromDate, mToDate);
@@ -1254,6 +1168,7 @@ void CalPrintMonth::setSettingsWidget()
         cfg->mColors->setChecked(mUseColors);
         cfg->mPrintFooter->setChecked(mPrintFooter);
         cfg->mIncludeDescription->setChecked(mIncludeDescription);
+        cfg->mIncludeCategories->setChecked(mIncludeCategories);
         cfg->mExcludeConfidential->setChecked(mExcludeConfidential);
         cfg->mExcludePrivate->setChecked(mExcludePrivate);
     }
@@ -1270,8 +1185,7 @@ void CalPrintMonth::loadConfig()
         mSingleLineLimit = grp.readEntry("Single line limit", false);
         mShowNoteLines = grp.readEntry("Note Lines", false);
         mIncludeDescription = grp.readEntry("Include description", false);
-        mExcludeConfidential = grp.readEntry("Exclude confidential", true);
-        mExcludePrivate = grp.readEntry("Exclude private", true);
+        mIncludeCategories = grp.readEntry("Include categories", false);
     }
     setSettingsWidget();
 }
@@ -1288,15 +1202,14 @@ void CalPrintMonth::saveConfig()
         grp.writeEntry("Single line limit", mSingleLineLimit);
         grp.writeEntry("Note Lines", mShowNoteLines);
         grp.writeEntry("Include description", mIncludeDescription);
-        grp.writeEntry("Exclude confidential", mExcludeConfidential);
-        grp.writeEntry("Exclude private", mExcludePrivate);
+        grp.writeEntry("Include categories", mIncludeCategories);
     }
 }
 
 void CalPrintMonth::setDateRange(const QDate &from, const QDate &to)
 {
     CalPrintPluginBase::setDateRange(from, to);
-    auto *cfg = dynamic_cast<CalPrintMonthConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintMonthConfig *>((QWidget *)mConfigWidget);
     if (cfg) {
         cfg->mFromMonth->clear();
         for (int i = 0; i < 12; ++i) {
@@ -1344,10 +1257,8 @@ void CalPrintMonth::print(QPainter &p, int width, int height)
                        mRecurDaily,
                        mRecurWeekly,
                        mSingleLineLimit,
-                       mShowNoteLines,
                        mIncludeDescription,
-                       mExcludeConfidential,
-                       mExcludePrivate,
+                       mIncludeCategories,
                        monthBox);
 
         if (mPrintFooter) {
@@ -1383,7 +1294,7 @@ QWidget *CalPrintTodos::createConfigWidget(QWidget *w)
 
 void CalPrintTodos::readSettingsWidget()
 {
-    auto *cfg = dynamic_cast<CalPrintTodoConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintTodoConfig *>((QWidget *)mConfigWidget);
 
     if (cfg) {
         mPageTitle = cfg->mTitle->text();
@@ -1403,6 +1314,8 @@ void CalPrintTodos::readSettingsWidget()
 
         mIncludeDescription = cfg->mDescription->isChecked();
         mIncludePriority = cfg->mPriority->isChecked();
+        mIncludeCategories = cfg->mCategories->isChecked();
+        mIncludeStartDate = cfg->mStartDate->isChecked();
         mIncludeDueDate = cfg->mDueDate->isChecked();
         mIncludePercentComplete = cfg->mPercentComplete->isChecked();
         mConnectSubTodos = cfg->mConnectSubTodos->isChecked();
@@ -1419,7 +1332,7 @@ void CalPrintTodos::readSettingsWidget()
 
 void CalPrintTodos::setSettingsWidget()
 {
-    auto *cfg = dynamic_cast<CalPrintTodoConfig *>((QWidget *)mConfigWidget);
+    auto cfg = dynamic_cast<CalPrintTodoConfig *>((QWidget *)mConfigWidget);
     if (cfg) {
         cfg->mTitle->setText(mPageTitle);
 
@@ -1432,6 +1345,8 @@ void CalPrintTodos::setSettingsWidget()
 
         cfg->mDescription->setChecked(mIncludeDescription);
         cfg->mPriority->setChecked(mIncludePriority);
+        cfg->mCategories->setChecked(mIncludeCategories);
+        cfg->mStartDate->setChecked(mIncludeStartDate);
         cfg->mDueDate->setChecked(mIncludeDueDate);
         cfg->mPercentComplete->setChecked(mIncludePercentComplete);
         cfg->mConnectSubTodos->setChecked(mConnectSubTodos);
@@ -1446,6 +1361,7 @@ void CalPrintTodos::setSettingsWidget()
             cfg->mSortField->addItem(i18nc("@option sort by due date/time", "Due Date"));
             cfg->mSortField->addItem(i18nc("@option sort by priority", "Priority"));
             cfg->mSortField->addItem(i18nc("@option sort by percent completed", "Percent Complete"));
+            cfg->mSortField->addItem(i18nc("@option sort by tags", "Tags"));
             cfg->mSortField->setCurrentIndex(mTodoSortField);
         }
 
@@ -1468,14 +1384,14 @@ void CalPrintTodos::loadConfig()
         mTodoPrintType = (eTodoPrintType)grp.readEntry("Print type", static_cast<int>(TodosAll));
         mIncludeDescription = grp.readEntry("Include description", true);
         mIncludePriority = grp.readEntry("Include priority", true);
+        mIncludeCategories = grp.readEntry("Include categories", true);
+        mIncludeStartDate = grp.readEntry("Include start date", true);
         mIncludeDueDate = grp.readEntry("Include due date", true);
         mIncludePercentComplete = grp.readEntry("Include percentage completed", true);
         mConnectSubTodos = grp.readEntry("Connect subtodos", true);
         mStrikeOutCompleted = grp.readEntry("Strike out completed summaries", true);
         mTodoSortField = (eTodoSortField)grp.readEntry("Sort field", static_cast<int>(TodoFieldSummary));
         mTodoSortDirection = (eTodoSortDirection)grp.readEntry("Sort direction", static_cast<int>(TodoDirectionAscending));
-        mExcludeConfidential = grp.readEntry("Exclude confidential", true);
-        mExcludePrivate = grp.readEntry("Exclude private", true);
     }
     setSettingsWidget();
 }
@@ -1489,20 +1405,19 @@ void CalPrintTodos::saveConfig()
         grp.writeEntry("Print type", int(mTodoPrintType));
         grp.writeEntry("Include description", mIncludeDescription);
         grp.writeEntry("Include priority", mIncludePriority);
+        grp.writeEntry("Include categories", mIncludeCategories);
+        grp.writeEntry("Include start date", mIncludeStartDate);
         grp.writeEntry("Include due date", mIncludeDueDate);
         grp.writeEntry("Include percentage completed", mIncludePercentComplete);
         grp.writeEntry("Connect subtodos", mConnectSubTodos);
         grp.writeEntry("Strike out completed summaries", mStrikeOutCompleted);
         grp.writeEntry("Sort field", static_cast<int>(mTodoSortField));
         grp.writeEntry("Sort direction", static_cast<int>(mTodoSortDirection));
-        grp.writeEntry("Exclude confidential", mExcludeConfidential);
-        grp.writeEntry("Exclude private", mExcludePrivate);
     }
 }
 
 void CalPrintTodos::print(QPainter &p, int width, int height)
 {
-    int pospriority = 0;
     int possummary = 100;
 
     QRect headerBox(0, 0, width, headerHeight());
@@ -1525,41 +1440,55 @@ void CalPrintTodos::print(QPainter &p, int width, int height)
     p.setFont(QFont(QStringLiteral("sans-serif"), 9, QFont::Bold));
     int lineSpacing = p.fontMetrics().lineSpacing();
     mCurrentLinePos += lineSpacing;
+    int pospriority = -1;
     if (mIncludePriority) {
         outStr += i18n("Priority");
+        pospriority = 0;
         p.drawText(pospriority, mCurrentLinePos - 2, outStr);
-    } else {
-        pospriority = -1;
     }
 
-    int posdue;
+    int posSoFar = width;  // Position of leftmost optional header.
+
+    int posdue = -1;
     if (mIncludeDueDate) {
         outStr.truncate(0);
         outStr += i18nc("@label to-do due date", "Due");
         const int widDue = std::max(p.fontMetrics().boundingRect(outStr).width(), widDate);
-        posdue = width - widDue;
+        posdue = posSoFar - widDue;
         p.drawText(posdue, mCurrentLinePos - 2, outStr);
-    } else {
-        posdue = -1;
+        posSoFar = posdue;
     }
 
-    int poscomplete;
+    int posStart = -1;
+    if (mIncludeStartDate) {
+        outStr.truncate(0);
+        outStr += i18nc("@label to-do start date", "Start");
+        const int widStart = std::max(p.fontMetrics().boundingRect(outStr).width(), widDate);
+        posStart = posSoFar - widStart - 5;
+        p.drawText(posStart, mCurrentLinePos - 2, outStr);
+        posSoFar = posStart;
+    }
+
+    int poscomplete = -1;
     if (mIncludePercentComplete) {
         outStr.truncate(0);
         outStr += i18nc("@label to-do percentage complete", "Complete");
         const int widComplete = std::max(p.fontMetrics().boundingRect(outStr).width(), widPct);
-        if (!mIncludeDueDate) {
-            poscomplete = width - widComplete;
-        } else {
-            poscomplete = posdue - widComplete - 5;
-        }
+        poscomplete = posSoFar - widComplete - 5;
         p.drawText(poscomplete, mCurrentLinePos - 2, outStr);
-    } else {
-        poscomplete = -1;
+        posSoFar = poscomplete;
+    }
+
+    int posCategories = -1;
+    if (mIncludeCategories) {
+        outStr.truncate(0);
+        outStr += i18nc("@label to-do categories", "Tags");
+        const int widCats = std::max(p.fontMetrics().boundingRect(outStr).width(), 100); // Arbitrary!
+        posCategories = posSoFar - widCats - 5;
+        p.drawText(posCategories, mCurrentLinePos - 2, outStr);
     }
 
     p.setFont(QFont(QStringLiteral("sans-serif"), 10));
-    // fontHeight = p.fontMetrics().height();
 
     KCalendarCore::Todo::List todoList;
     KCalendarCore::Todo::List tempList;
@@ -1593,6 +1522,9 @@ void CalPrintTodos::print(QPainter &p, int width, int height)
     case TodoFieldPercentComplete:
         sortField = KCalendarCore::TodoSortPercentComplete;
         break;
+    case TodoFieldCategories:
+        sortField = KCalendarCore::TodoSortCategories;
+        break;
     case TodoFieldUnset:
         break;
     }
@@ -1603,7 +1535,7 @@ void CalPrintTodos::print(QPainter &p, int width, int height)
     case TodosAll:
         break;
     case TodosUnfinished:
-        for (const KCalendarCore::Todo::Ptr &todo : qAsConst(todoList)) {
+        for (const KCalendarCore::Todo::Ptr &todo : std::as_const(todoList)) {
             Q_ASSERT(todo);
             if (!todo->isCompleted()) {
                 tempList.append(todo);
@@ -1612,7 +1544,7 @@ void CalPrintTodos::print(QPainter &p, int width, int height)
         todoList = tempList;
         break;
     case TodosDueRange:
-        for (const KCalendarCore::Todo::Ptr &todo : qAsConst(todoList)) {
+        for (const KCalendarCore::Todo::Ptr &todo : std::as_const(todoList)) {
             Q_ASSERT(todo);
             if (todo->hasDueDate()) {
                 if (todo->dtDue().date() >= mFromDate && todo->dtDue().date() <= mToDate) {
@@ -1628,7 +1560,7 @@ void CalPrintTodos::print(QPainter &p, int width, int height)
 
     // Print to-dos
     int count = 0;
-    for (const KCalendarCore::Todo::Ptr &todo : qAsConst(todoList)) {
+    for (const KCalendarCore::Todo::Ptr &todo : std::as_const(todoList)) {
         if ((mExcludeConfidential && todo->secrecy() == KCalendarCore::Incidence::SecrecyConfidential)
             || (mExcludePrivate && todo->secrecy() == KCalendarCore::Incidence::SecrecyPrivate)) {
             continue;
@@ -1646,6 +1578,8 @@ void CalPrintTodos::print(QPainter &p, int width, int height)
                      mIncludeDescription,
                      pospriority,
                      possummary,
+                     posCategories,
+                     posStart,
                      posdue,
                      poscomplete,
                      0,
@@ -1654,9 +1588,7 @@ void CalPrintTodos::print(QPainter &p, int width, int height)
                      width,
                      height,
                      todoList,
-                     nullptr,
-                     mExcludeConfidential,
-                     mExcludePrivate);
+                     nullptr);
         }
     }
 
