@@ -7,8 +7,7 @@
   SPDX-License-Identifier: GPL-2.0-or-later WITH Qt-Commercial-exception-1.0
 */
 
-#ifndef CALENDARSUPPORT_PRINTING_CALPRINTPLUGINBASE_H
-#define CALENDARSUPPORT_PRINTING_CALPRINTPLUGINBASE_H
+#pragma once
 
 #include "calendarsupport_export.h"
 #include "printplugin.h"
@@ -25,9 +24,9 @@
 class PrintCellItem;
 class QWidget;
 
-#define PORTRAIT_HEADER_HEIGHT 72 // header height, for portrait orientation
+#define PORTRAIT_HEADER_HEIGHT 80 // header height, for portrait orientation
 #define LANDSCAPE_HEADER_HEIGHT 54 // header height, for landscape orientation
-#define SUBHEADER_HEIGHT 20 // subheader height, for all orientations
+#define SUBHEADER_HEIGHT 28 // subheader height, for all orientations
 #define PORTRAIT_FOOTER_HEIGHT 16 // footer height, for portrait orientation
 #define LANDSCAPE_FOOTER_HEIGHT 14 // footer height, for landscape orientation
 #define MARGIN_SIZE 36 // margins, for all orientations
@@ -145,7 +144,7 @@ public:
       @param linewidth The border width of the box
       @param rect The rectangle of the box
     */
-    static void drawBox(QPainter &p, int linewidth, const QRect &rect);
+    static void drawBox(QPainter &p, int linewidth, QRect rect);
     /**
       Draw a shaded box with given width at the given coordinates.
       @param p The printer to be used
@@ -153,7 +152,7 @@ public:
       @param brush The brush to fill the box
       @param rect The rectangle of the box
     */
-    static void drawShadedBox(QPainter &p, int linewidth, const QBrush &brush, const QRect &rect);
+    static void drawShadedBox(QPainter &p, int linewidth, const QBrush &brush, QRect rect);
 
     /**
       Print the given string (event summary) in the given rectangle. Margins
@@ -162,7 +161,8 @@ public:
       @param box Coordinates of the surrounding event box
       @param str The text to be printed in the box
     */
-    void printEventString(QPainter &p, const QRect &box, const QString &str, int flags = -1);
+    void printEventString(QPainter &p, QRect box, const QString &str, int flags = -1);
+
     /**
       Print the box for the given event with the given string.
       @param p QPainter of the printout
@@ -173,7 +173,7 @@ public:
       @param str The string to print inside the box
       @param flags is a bitwise OR of Qt::AlignmentFlags and Qt::TextFlags values.
     */
-    void showEventBox(QPainter &p, int linewidth, const QRect &box, const KCalendarCore::Incidence::Ptr &incidence, const QString &str, int flags = -1);
+    void showEventBox(QPainter &p, int linewidth, QRect box, const KCalendarCore::Incidence::Ptr &incidence, const QString &str, int flags = -1);
 
     /**
       Draw a subheader box with a shaded background and the given string
@@ -181,7 +181,7 @@ public:
       @param str Text to be printed inside the box
       @param box Coordinates of the box
     */
-    void drawSubHeaderBox(QPainter &p, const QString &str, const QRect &box);
+    void drawSubHeaderBox(QPainter &p, const QString &str, QRect box);
 
     /**
       Draw an event box with vertical text.
@@ -191,7 +191,7 @@ public:
       @param str ext to be printed inside the box
       @param flags is a bitwise OR of Qt::AlignmentFlags and Qt::TextFlags values.
     */
-    void drawVerticalBox(QPainter &p, int linewidth, const QRect &box, const QString &str, int flags = -1);
+    void drawVerticalBox(QPainter &p, int linewidth, QRect box, const QString &str, int flags = -1);
 
     /**
       Draw a component box with a heading (printed in bold).
@@ -215,7 +215,7 @@ public:
               custom contents in that case.
     */
     int drawBoxWithCaption(QPainter &p,
-                           const QRect &box,
+                           QRect box,
                            const QString &caption,
                            const QString &contents,
                            bool sameLine,
@@ -252,8 +252,7 @@ public:
               is box.bottom, otherwise it is larger than box.bottom
               and matches the y-coordinate of the surrounding rectangle.
     */
-    int
-    drawHeader(QPainter &p, const QString &title, const QDate &month1, const QDate &month2, const QRect &box, bool expand = false, QColor backColor = QColor());
+    int drawHeader(QPainter &p, const QString &title, QDate month1, QDate month2, QRect box, bool expand = false, QColor backColor = QColor());
 
     /**
       Draw a page footer containing the printing date and possibly
@@ -262,7 +261,7 @@ public:
       @param box coordinates of the footer
       @return The bottom of the printed box.
     */
-    int drawFooter(QPainter &p, const QRect &box);
+    int drawFooter(QPainter &p, QRect box);
 
     /**
       Draw a small calendar with the days of a month into the given area.
@@ -271,7 +270,7 @@ public:
       @param qd Arbitrary Date within the month to be printed.
       @param box coordinates of the small calendar
     */
-    void drawSmallMonth(QPainter &p, const QDate &qd, const QRect &box);
+    void drawSmallMonth(QPainter &p, QDate qd, QRect box);
 
     /**
       Draw a horizontal bar with the weekday names of the given date range
@@ -282,7 +281,8 @@ public:
       @param toDate Last date of the printed dates
       @param box coordinates of the box for the days of the week
     */
-    void drawDaysOfWeek(QPainter &p, const QDate &fromDate, const QDate &toDate, const QRect &box);
+    void drawDaysOfWeek(QPainter &p, const QDate &fromDate, QDate toDate, QRect box);
+
     /**
       Draw a single weekday name in a box inside the given area of the painter.
       This is called in a loop by drawDaysOfWeek.
@@ -290,7 +290,7 @@ public:
       @param qd Date of the printed day
       @param box coordinates of the weekbox
     */
-    void drawDaysOfWeekBox(QPainter &p, const QDate &qd, const QRect &box);
+    void drawDaysOfWeekBox(QPainter &p, QDate qd, QRect box);
 
     /**
       Draw a (vertical) time scale from time fromTime to toTime inside the
@@ -302,40 +302,34 @@ public:
       @param toTime End time of the time range to display
       @param box coordinates of the timeline
     */
-    void drawTimeLine(QPainter &p, const QTime &fromTime, const QTime &toTime, const QRect &box);
+    void drawTimeLine(QPainter &p, QTime fromTime, QTime toTime, QRect box);
 
     /**
       Draw the all-day box for the agenda print view (the box on top which
-      doesn't have a time on the time scale associated). If expandable is set,
-      height is the cell height of a single cell, and the returned height will
-      be the total height used for the all-day events. If !expandable, only one
-      cell will be used, and multiple events are concatenated using ", ".
+      doesn't have a time on the time scale associated).
+
+      Obeys configuration options #mExcludeConfidential, #excludePrivate.
       @param p QPainter of the printout
       @param eventList The list of all-day events that are supposed to be printed
              inside this box
       @param qd The date of the currently printed day
-      @param expandable If true, height is the height of one single cell, the printout
-             will use as many cells as events in the list and return the total height
-             needed for all of them. If false, height specifies the total height
-             allowed for all events, and the events are displayed in one cell,
-             with their summaries concatenated by ", ".
       @param box coordinates of the all day box.
-      @param excludeConfidential Whether to exclude Incidence marked confidential.
-      @param excludePrivate Whether to exclude Incidence marked private.
-      @return The height used for the all-day box.
+      @param includeCategories Whether to print the event categories (tags) as well.
+      @param workDays List of workDays
     */
-    int drawAllDayBox(QPainter &p,
+    void drawAllDayBox(QPainter &p,
                       const KCalendarCore::Event::List &eventList,
-                      const QDate &qd,
-                      bool expandable,
-                      const QRect &box,
-                      bool excludeConfidential,
-                      bool excludePrivate);
+                      QDate qd,
+                      QRect box,
+                      bool includeCategories,
+                      const QList<QDate> &workDays);
+
     /**
       Draw the agenda box for the day print style (the box showing all events of that day).
       Also draws a grid with half-hour spacing of the grid lines.
       Does NOT draw allday events.  Use drawAllDayBox for allday events.
 
+      Obeys configuration options #mExcludeConfidential, #excludePrivate.
       @param p QPainter of the printout
       @param eventList The list of the events that are supposed to be printed
              inside this box
@@ -352,9 +346,8 @@ public:
                    to include all events if expandable==true
       @param box coordinates of the agenda day box.
       @param includeDescription Whether to print the event description as well.
+      @param includeCategories Whether to print the event categories (tags) as well.
       @param excludeTime Whether the time is printed in the detail area.
-      @param excludeConfidential Whether to exclude Incidence marked confidential.
-      @param excludePrivate Whether to exclude Incidence marked private.
       @param workDays List of workDays
     */
     void drawAgendaDayBox(QPainter &p,
@@ -365,9 +358,8 @@ public:
                           const QTime &toTime,
                           const QRect &box,
                           bool includeDescription,
+                          bool includeCategories,
                           bool excludeTime,
-                          bool excludeConfidential,
-                          bool excludePrivate,
                           const QList<QDate> &workDays);
 
     void drawAgendaItem(PrintCellItem *item,
@@ -377,11 +369,14 @@ public:
                         float minlen,
                         const QRect &box,
                         bool includeDescription,
+                        bool includeCategories,
                         bool excludeTime);
 
     /**
       Draw the box containing a list of all events of the given day (with their times,
       of course). Used in the Filofax and the month print style.
+
+      Obeys configuration options #mExcludeConfidential, #mExcludePrivate, #mShowNoteLines, #mUseColors.
       @param p QPainter of the printout
       @param qd The date of the currently printed day. All events of the calendar
                 that appear on that day will be printed.
@@ -393,10 +388,8 @@ public:
       @param printRecurDaily Whether daily recurring incidences should be printed.
       @param printRecurWeekly Whether weekly recurring incidences should be printed.
       @param singleLineLimit Whether Incidence text wraps or truncates.
-      @param showNoteLines Whether note lines are printed.
       @param includeDescription Whether to print the event description as well.
-      @param excludeConfidential Whether to exclude Incidence marked confidential.
-      @param excludePrivate Whether to exclude Incidence marked private.
+      @param includeCategories Whether to print the event categories (tags) as well.
     */
     void drawDayBox(QPainter &p,
                     const QDate &qd,
@@ -407,25 +400,24 @@ public:
                     bool printRecurDaily = true,
                     bool printRecurWeekly = true,
                     bool singleLineLimit = true,
-                    bool showNoteLines = false,
                     bool includeDescription = false,
-                    bool excludeDescription = true,
-                    bool excludePrivate = true);
+                    bool includeCategories = false);
+
     /**
       Draw the week (filofax) table of the week containing the date qd. The first
       three days of the week will be shown in the first column (using drawDayBox),
       the remaining four in the second column, where the last two days of the week
       (typically Saturday and Sunday) only get half the height of the other day boxes.
+
+      Obeys configuration options #mExcludeConfidential, #mExcludePrivate, #mShowNoteLines, #mUseColors.
       @param p QPainter of the printout
       @param qd Arbitrary date within the week to be printed.
       @param fromTime Start time of the displayed time range
       @param toTime End time of the displayed time range
       @param box coordinates of the week box.
       @param singleLineLimit Whether Incidence text wraps or truncates.
-      @param showNoteLines Whether note lines are printed.
       @param includeDescription Whether to print the event description as well.
-      @param excludeConfidential Whether to exclude Incidence marked confidential.
-      @param excludePrivate Whether to exclude Incidence marked private.
+      @param includeCategories Whether to print the event categories (tags) as well.
     */
     void drawWeek(QPainter &p,
                   const QDate &qd,
@@ -433,12 +425,13 @@ public:
                   const QTime &toTime,
                   const QRect &box,
                   bool singleLineLimit,
-                  bool showNoteLines,
                   bool includeDescription,
-                  bool excludeConfidential,
-                  bool excludePrivate);
+                  bool includeCategories);
+
     /**
       Draw the (filofax) table for a bunch of days, using drawDayBox.
+
+      Obeys configuration options #mExcludeConfidential, #mExcludePrivate, #mShowNoteLines, #mUseColors.
       @param p QPainter of the printout
       @param start Start date
       @param end End date
@@ -446,10 +439,8 @@ public:
       @param toTime End time of the displayed time range
       @param box coordinates of the week box.
       @param singleLineLimit Whether Incidence text wraps or truncates.
-      @param showNoteLines Whether note lines are printed.
       @param includeDescription Whether to print the event description as well.
-      @param excludeConfidential Whether to exclude Incidence marked confidential.
-      @param excludePrivate Whether to exclude Incidence marked private.
+      @param includeCategories Whether to print the event categories (tags) as well.
     */
     void drawDays(QPainter &p,
                   const QDate &start,
@@ -458,10 +449,9 @@ public:
                   const QTime &toTime,
                   const QRect &box,
                   bool singleLineLimit,
-                  bool showNoteLines,
                   bool includeDescription,
-                  bool excludeConfidential,
-                  bool excludePrivate);
+                  bool includeCategories);
+
     /**
       Draw the timetable view of the given time range from fromDate to toDate.
       On the left side the time scale is printed (using drawTimeLine), then each
@@ -469,6 +459,8 @@ public:
       and the events are displayed as boxes (like in korganizer's day/week view).
       The first cell of each column contains the all-day events (using
       drawAllDayBox with expandable=false).
+
+      Obeys configuration options #mExcludeConfidential, #excludePrivate.
       @param p QPainter of the printout
       @param fromDate First day to be included in the page
       @param toDate Last day to be included in the page
@@ -478,9 +470,8 @@ public:
       @param toTime End time of the displayed time range
       @param box coordinates of the time table.
       @param includeDescription Whether to print the event description as well.
+      @param includeCategories Whether to print the event categories (tags) as well.
       @param excludeTime Whether the time is printed in the detail area.
-      @param excludeConfidential Whether to exclude Incidence marked confidential.
-      @param excludePrivate Whether to exclude Incidence marked private.
     */
     void drawTimeTable(QPainter &p,
                        const QDate &fromDate,
@@ -490,9 +481,8 @@ public:
                        const QTime &toTime,
                        const QRect &box,
                        bool includeDescription,
-                       bool excludeTime,
-                       bool excludeConfidential,
-                       bool excludePrivate);
+                       bool includeCategories,
+                       bool excludeTime);
 
     /**
       Draw the month table of the month containing the date qd. Each day gets one
@@ -500,6 +490,8 @@ public:
       in a matrix, with the first column being the first day of the
       week (so it might display some days of the previous and the next month).
       Above the matrix there is a bar showing the weekdays (drawn using drawDaysOfWeek).
+
+      Obeys configuration options #mExcludeConfidential, #mExcludePrivate, #mShowNoteLines, #mUseColors.
       @param p QPainter of the printout
       @param qd Arbitrary date within the month to be printed.
       @param fromTime Start time of the displayed time range
@@ -508,10 +500,8 @@ public:
       @param recurDaily Whether daily recurring incidences should be printed.
       @param recurWeekly Whether weekly recurring incidences should be printed.
       @param singleLineLimit Whether Incidence text wraps or truncates.
-      @param showNoteLines Whether note lines are printed.
       @param includeDescription Whether descriptions are printed.
-      @param excludeConfidential Whether to exclude Incidence marked confidential.
-      @param excludePrivate Whether to exclude Incidence marked private.
+      @param includeCategories Whether to print the event categories (tags) as well.
       @param box coordinates of the month.
     */
     void drawMonthTable(QPainter &p,
@@ -522,14 +512,15 @@ public:
                         bool recurDaily,
                         bool recurWeekly,
                         bool singleLineLimit,
-                        bool showNoteLines,
                         bool includeDescription,
-                        bool excludeConfidential,
-                        bool excludePrivate,
+                        bool includeCategories,
                         const QRect &box);
+
     /**
       Draw a vertical representation of the month containing the date dt. Each
       day gets one line.
+
+      Obeys configuration options #mExcludeConfidential, #excludePrivate.
       @param p QPainter of the printout
       @param dt Arbitrary date within the month to be printed
       @param box coordinates of the box reserved for the month
@@ -541,17 +532,13 @@ public:
                            how events that do not cross midnight should be printed.
       @param holidaysFlags Bitfield consisting of DisplayFlags flags to determine
                            how holidays should be printed.
-      @param excludeConfidential Whether to exclude Incidence marked confidential.
-      @param excludePrivate Whether to exclude Incidence marked private.
     */
     void drawMonth(QPainter &p,
                    const QDate &dt,
                    const QRect &box,
                    int maxdays = -1,
                    int subDailyFlags = TimeBoxes,
-                   int holidaysFlags = Text,
-                   bool excludeConfidential = false,
-                   bool excludePrivate = false);
+                   int holidaysFlags = Text);
 
     /**
       Internal class representing the start of a todo.
@@ -559,7 +546,7 @@ public:
     class TodoParentStart;
 
     /**
-      Draws single to-do and its (intented) sub-to-dos, optionally connects them
+      Draws single to-do and its (indented) sub-to-dos, optionally connects them
       by a tree-like line, and optionally shows due date, summary, description
       and priority.
       @param count The number of the currently printed to-do (count will be
@@ -576,13 +563,17 @@ public:
       @param desc Whether to print the whole description of the to-do
       (the summary is always printed).
       @param posPriority x-coordinate where the priority is supposed to be
-      printed. If <0, no priority will be printed.
+      printed. If negative, no priority will be printed.
       @param posSummary x-coordinate where the summary of the to-do is supposed
       to be printed.
+      @param posCategories x-coordinate where the categories (tags) should be
+      printed. If negative, no categories will be printed.
+      @param posStartDt x-coordinate where the due date is supposed to the be
+      printed. If negative, no start date will be printed.
       @param posDueDt x-coordinate where the due date is supposed to the be
-      printed. If <0, no due date will be printed.
+      printed. If negative, no due date will be printed.
       @param posPercentComplete x-coordinate where the percentage complete is
-      supposed to be printed. If <0, percentage complete will not be printed.
+      supposed to be printed. If negative, percentage complete will not be printed.
       @param level Level of the current to-do in the to-do hierarchy (0 means
       highest level of printed to-dos, 1 are their sub-to-dos, etc.)
       @param x x-coordinate of the upper left coordinate of the first to-do.
@@ -604,6 +595,8 @@ public:
                   bool desc,
                   int posPriority,
                   int posSummary,
+                  int posCategories,
+                  int posStartDt,
                   int posDueDt,
                   int posPercentComplete,
                   int level,
@@ -612,21 +605,8 @@ public:
                   int width,
                   int pageHeight,
                   const KCalendarCore::Todo::List &todoList,
-                  TodoParentStart *r,
-                  bool excludeConfidential,
-                  bool excludePrivate);
+                  TodoParentStart *r);
 
-    /**
-      Draws single journal item.
-      @param journal The item to be printed.
-      @param p QPainter of the printout
-      @param x x-coordinate of the upper left coordinate of the first item
-      @param y y-coordinate of the upper left coordinate of the first item
-      @param width width of the whole list
-      @param pageHeight Total height allowed for the list on a page. If an item
-                   would be below that line, a new page is started.
-    */
-    void drawJournal(const KCalendarCore::Journal::Ptr &journal, QPainter &p, int x, int &y, int width, int pageHeight);
     /**
       Draws text lines splitting on page boundaries.
       @param p QPainter of the printout
@@ -638,7 +618,7 @@ public:
     */
     void drawTextLines(QPainter &p, const QString &entry, int x, int &y, int width, int pageHeight, bool richTextEntry);
 
-    void drawSplitHeaderRight(QPainter &p, const QDate &fd, const QDate &td, const QDate &cd, int width, int height);
+    void drawSplitHeaderRight(QPainter &p, QDate fd, QDate td, QDate cd, int width, int height);
 
     /**
       Draws dotted lines for notes in a box.
@@ -646,7 +626,7 @@ public:
       @param box coordinates of the box where the lines will be placed
       @param startY starting y-coordinate for the first line
     */
-    void drawNoteLines(QPainter &p, const QRect &box, int startY);
+    void drawNoteLines(QPainter &p, QRect box, int startY);
 
 protected:
     QTime dayStart() const;
@@ -673,9 +653,11 @@ protected:
                        bool connectSubTodos);
 
 protected:
-    bool mUseColors;
-    bool mPrintFooter;
-    bool mShowNoteLines;
+    bool mUseColors;    /**< Whether or not to use event category colors to draw the events. */
+    bool mPrintFooter;  /**< Whether or not to print a footer at the bottoms of pages. */
+    bool mShowNoteLines;    /**< Whether or not to print horizontal  lines in note areas. */
+    bool mExcludeConfidential;  /**< Whether or not to print incidences with secrecy "confidential". */
+    bool mExcludePrivate;   /**< Whether or not to print incidences with secrecy "private". */
     int mHeaderHeight;
     int mSubHeaderHeight;
     int mFooterHeight;
@@ -691,9 +673,9 @@ private:
      */
     void setColorsByIncidenceCategory(QPainter &p, const KCalendarCore::Incidence::Ptr &incidence) const;
 
-    QString holidayString(const QDate &date) const;
+    QString holidayString(QDate date) const;
 
-    KCalendarCore::Event::Ptr holidayEvent(const QDate &date) const;
+    KCalendarCore::Event::Ptr holidayEvent(QDate date) const;
 
     /**
      * Returns a nice QColor for text, give the input color &c.
@@ -702,4 +684,3 @@ private:
 };
 }
 
-#endif
